from pyogre import ogre
import SampleFramework

class TutorialFrameListener(SampleFramework.FrameListener):
	def __init__(self, renderWindow, camera, sceneManager):
		SampleFramework.FrameListener.__init__(self, renderWindow, camera)
		
		self.mouseDown = False
		self.toggle = 0

		# populate the camera and scene manager containers
		self.camNode = camera.parentSceneNode.parentSceneNode
		self.sceneManager = sceneManager

		# set the rotation and movement speed
		self.rotate = 0.05
		self.moveSpeed= 250

	def frameStarted(self, evt):
		
		inputDevice = self.inputDevice
		inputDevice.capture()
		
		currMouse = inputDevice.getMouseButton(0)
		if currMouse and not self.mouseDown:
			light = self.sceneManager.getLight("Light1")
			light.visible = not light.visible
		self.mouseDown = currMouse
		
		if self.toggle >= 0:
			self.toggle -= evt.timeSinceLastFrame
			
		if self.toggle < 0 and inputDevice.isKeyDown(ogre.KC_1):
			self.toggle = 1.0
			
			self.camera.parentSceneNode.detachObject(self.camera)
			self.camNode = self.sceneManager.getSceneNode("CamNode1")
			self.sceneManager.getSceneNode("PitchNode1").attachObject(self.camera)
		
		elif self.toggle < 0 and inputDevice.isKeyDown(ogre.KC_2):
			self.toggle = 1.0

			self.camera.parentSceneNode.detachObject(self.camera)
			self.camNode = self.sceneManager.getSceneNode("CamNode2")
			self.sceneManager.getSceneNode("PitchNode2").attachObject(self.camera)
			
		transVector = ogre.Vector3(0, 0, 0)
		if inputDevice.isKeyDown(ogre.KC_UP) or inputDevice.isKeyDown(ogre.KC_W):
			transVector.z -= self.moveSpeed
		if inputDevice.isKeyDown(ogre.KC_DOWN) or inputDevice.isKeyDown(ogre.KC_S):
			transVector.z += self.moveSpeed
		if inputDevice.isKeyDown(ogre.KC_LEFT) or inputDevice.isKeyDown(ogre.KC_A):
			transVector.x -= self.moveSpeed
		if inputDevice.isKeyDown(ogre.KC_RIGHT) or inputDevice.isKeyDown(ogre.KC_D):
			transVector.x += self.moveSpeed
		if inputDevice.isKeyDown(ogre.KC_PGUP) or inputDevice.isKeyDown(ogre.KC_Q):
			transVector.y += self.moveSpeed
		if inputDevice.isKeyDown(ogre.KC_PGDOWN) or inputDevice.isKeyDown(ogre.KC_E):
			transVector.y -= self.moveSpeed
			
		self.camNode.translate(self.camNode.orientation * transVector * evt.timeSinceLastFrame)
		if inputDevice.getMouseButton(2):
			self.camNode.yaw(ogre.Degree(-self.rotate * inputDevice.mouseRelativeX))
			self.camNode.getChild(0).pitch(ogre.Degree(-self.rotate * self.inputDevice.mouseRelativeY))
		
		
		return SampleFramework.FrameListener.frameStarted(self, evt)

		return not inputDevice.isKeyDown(ogre.KC_ESCAPE)
	
class TutorialApplication(SampleFramework.Application):
	def _createScene(self):
		pass
		sceneManager = self.sceneManager
		sceneManager.ambientLight = 0.1, 0.1, 0.1
		
		ent = sceneManager.createEntity("Ninja", "ninja.mesh")
		node = sceneManager.rootSceneNode.createChildSceneNode("NinjaNode")
		node.attachObject(ent)
		
		light = sceneManager.createLight("Light1")
		light.type = ogre.Light.LT_POINT
		light.position = 250, 250, 250
		light.diffuseColour = 1, 1, 1
		light.specularColour =1, 1,1
		
		# create the first camera node/pitch node
		node = sceneManager.rootSceneNode.createChildSceneNode("CamNode1", (-400, 200, 400))
		node.yaw(ogre.Degree(-45))  # look at the ninja
		
		node = node.createChildSceneNode("PitchNode1")
		node.attachObject(self.camera)
		
		# create the second camera node/pitch node
		node = sceneManager.rootSceneNode.createChildSceneNode("CamNode2", (0, 200, 400))
		node.createChildSceneNode("PitchNode2")

	def _createCamera(self):
		pass
		self.camera = self.sceneManager.createCamera("PlayerCam")
		self.camera.nearClipDistance = 5

	def _createFrameListener(self):
		pass
		self.frameListener = TutorialFrameListener(self.renderWindow, self.camera, self.sceneManager)
		self.root.addFrameListener(self.frameListener)
		self.frameListener.showDebugOverlay(True)
 
if __name__ == '__main__':
	try:
		ta = TutorialApplication()
		ta.go()
	except ogre.OgreException, e:
		print e


